package de.osci.osci12.samples;

import de.osci.osci12.roles.*;
import de.osci.osci12.common.*;
import de.osci.osci12.messagetypes.*;
import de.osci.osci12.messageparts.*;
import java.util.*;
import java.io.*;

/**
 * This is a demo application for a synchronous communication scenario
 * (request-response) according to the OSCI 1.2-trensport specification.
 * The main method's first parameter is the intermediary's URL, the second one
 * is the recipient's URL.
 *
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Organisation: </p>
 * @author N.Bngener
 * @version 0.9
 */

public class RequestResponse
{

  Intermed intermed;

  public RequestResponse(String intermedURL) throws Exception
  {
    // Install a security provider
    java.security.Security.addProvider(new org.bouncycastle.jce.provider.BouncyCastleProvider());

    // First we need the Intermediarys Cipher-Certificate
    java.security.cert.X509Certificate intermedCipherCert = de.osci.helper.Tools.createCertificate(getClass().getResourceAsStream("/de/osci/osci12/samples/zertifikate/Intermed.cer"));

    // Create the Intermed-role object
    intermed = new Intermed(null, intermedCipherCert, new java.net.URI(intermedURL));
  }

  public ResponseToMediateDelivery sendMediateDelivery(String urlRecipient) throws Exception
  {
    // Create the Originator-object. Since we have the private key(s), we use this
    // Constructor. We need a Decrypter-object for decryption of response
    Originator user_1 = new Originator(new de.osci.osci12.samples.impl.crypto.PKCS12Signer("/de/osci/osci12/samples/zertifikate/Sender1.p12", "123456"),
                                new de.osci.osci12.samples.impl.crypto.PKCS12Decrypter("/de/osci/osci12/samples/zertifikate/Sender1.p12", "123456"));

    // Create a DialogHandler
    // Default configuration is transport encryption and signature activated
    DialogHandler clientDialog = new DialogHandler(user_1, intermed,
        new de.osci.osci12.samples.impl.HttpTransport());

    // Create the GetMessageId-Request
    GetMessageId getMsgID = new GetMessageId(clientDialog);

    // Transmit the request and receive the response
    ResponseToGetMessageId rsp2GetMsgID = getMsgID.send();

    // Error handling
    if (!rsp2GetMsgID.getFeedback()[0][1].startsWith("0"))
    {
      // do error handling
    }

    // Now we have a message-Id, we are ready to send the MediateDelivery-request

    // First we create the Addressee-object representing the user 2.
    // The signature certificate is not needed since we do not
    // expect a signed response from the addressee in this secenario
    // (it would be included in a signed response anyway).
    Addressee user_2 = new Addressee(null,
        de.osci.helper.Tools.createCertificate(getClass().
        getResourceAsStream("/de/osci/osci12/samples/zertifikate/Empfaenger.cer")));

    // Create the MediateDelivery-message object
    // The constructor's second parameter (Addressee-object) may be null.
    // In this case you have to disable transport encryption calling
    // clientDialog.setEncryption(false);
    // This is usually not recommended. As a side effect, the recipient will not
    // be able to fetch the process card later from the intermediary.
    MediateDelivery mediateDel = new MediateDelivery(clientDialog, user_2, urlRecipient);

    // If the next line is removed, we will have no recording
    mediateDel.setMessageId(rsp2GetMsgID.getMessageId());
    mediateDel.setSubject("Subject");

    // We trust the intermediary's time stamp
    mediateDel.setQualityOfTimeStampCreation(false);
    mediateDel.setQualityOfTimeStampReception(false);

    // Add Content-Data
    ContentContainer data = new ContentContainer();
    data.addContent(new Content("Any content data."));
    mediateDel.addContentContainer(data);

    // Transmit the request and receive the response
    ResponseToMediateDelivery rsp2MedDel = mediateDel.send();
    if (!rsp2MedDel.getFeedback()[0][1].startsWith("0"))
    {
      // Error handling
    }
    // Optional: Check process card bundle (result of certificate check etc.)
//    rsp2MedDel.getProcessCardBundleRequest();
//    rsp2MedDel.getProcessCardBundleReply();
    // ....

    return rsp2MedDel;
  }

  public static void main(String[] args) {
    try {

      RequestResponse scenario_3 = new RequestResponse(args[0]);

      ResponseToMediateDelivery responseMediate = scenario_3.sendMediateDelivery(args[1]);
      System.out.println("\nResponseToStoreDelivery:\n"+responseMediate.toString());

      System.out.println("\nInhaltsdaten:\n"+responseMediate.getContentContainer()[0].getContents()[0].getContentData());


    }
    catch (Exception ex) {
      ex.printStackTrace();
    }
  }
}