package de.osci.osci12.samples.impl.crypto;

import de.osci.osci12.extinterfaces.crypto.*;
import de.osci.osci12.samples.impl.crypto.*;

import org.apache.commons.logging.*;

import java.io.*;

import java.security.*;
import java.security.cert.*;

import java.util.Enumeration;


/**
 * Diese Klasse ist eine Beispiel-Implementierung der abstrakten Decrypter-Klasse.
 * Fr die Verwendung wird ein PKCS#12-Keystore in Form einer *.p12-Datei bentigt.
 * Die Implementierung ist fr Testzwecke bestimmt, sie greift auf den ersten
 * verfgbaren Alias zu. Die PIN fr dessen Privatschlssel mu die gleiche sein
 * wie die des Keystores.
 *
 * Diese einfache Implementierung hlt die PIN des Keystores als Character-Array
 * im Arbeitsspeicher, sie wird als String bergeben. Es wird Anwendern empfohlen,
 * eigene Implementierungen zu schreiben, die die PIN in der Methode decrypt(...)
 * abfragen und nach Gebrauch wieder lschen oder anderweitig fr ein sicheres
 * Pin-Cashing zu sorgen.
 *
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Organisation: </p>
 * @author R. Lindemann, N.Bngener
 * @version 1.0
 * @see de.osci.osci12.extinterfaces.crypto.Decrypter
 */
public class PKCS12Decrypter extends de.osci.osci12.extinterfaces.crypto.Decrypter
{
  private static Log log = LogFactory.getLog(PKCS12Decrypter.class);
  private X509Certificate cert;
  private String alias;
  private char[] pin = null;
  private KeyStore keyStore = null;
  private Key key;

  //  private CertificateStoreProviderI certificateStoreProvider;
  public PKCS12Decrypter(String p12_fileName, String pin) throws Exception
  {
    this.pin = pin.toCharArray();
    keyStore = java.security.KeyStore.getInstance("PKCS12");
    keyStore.load(getClass().getResourceAsStream(p12_fileName), this.pin);
    this.alias = (String) keyStore.aliases().nextElement();

    cert = (X509Certificate) keyStore.getCertificate(alias);
    key = keyStore.getKey(alias, this.pin);
  }

  /**
   * Liefert die Versionsnummer
   * @return Versionsnummer
   */
  public String getVendor(){return "BOS";}

  /**
   * Sollte den Namen des Herstellers zurckgeben.
   * @return Herstellername
   */
  public String getVersion(){return "0.9";}

  /**
   *  Liefert das Verschlsselungszertifikat.
   *
   *@return  das Verschlsselungszertifikat
   */
  public X509Certificate getCertificate()
  {
     return cert;
  }

  /**
   *  Entschlsselt das bergebene Byte-Array
   *
   *@param  data die zu entschlsselnden Daten
   *@return die verschlsselten Daten
   */
    public byte[] decrypt(byte[] data) throws de.osci.osci12.encryption.OSCICipherException
    {
        try
        {
          log.debug("data: "+new String (data));
          return de.osci.osci12.encryption.Crypto.doRSADecryption(key, data);
        }
        catch (Exception ex)
        {
          log.error("Fehler: ",ex);
          throw new de.osci.osci12.encryption.OSCICipherException("Problem beim Entschlsseln - " + ex.getClass() + ": " + ex.getMessage());
        }
    }

}
