package de.osci.osci12.samples.impl.crypto;

import de.osci.osci12.extinterfaces.crypto.*;
import de.osci.osci12.samples.impl.crypto.*;

import org.apache.commons.logging.*;

import java.io.*;

import java.security.*;
import java.security.cert.*;

import java.util.Enumeration;
import de.osci.osci12.common.*;


/**
 * Diese Klasse ist eine Beispiel-Implementierung der abstrakten Signer-Klasse.
 * Fr die Verwendung wird ein PKCS#12-Keystore in Form einer *.p12-Datei bentigt.
 * Die Implementierung ist fr Testzwecke bestimmt, sie greift auf den ersten
 * verfgbaren Alias zu. Die PIN fr dessen Privatschlssel mu die gleiche sein
 * wie die des Keystores.
 *
 * Diese einfache Implementierung hlt die PIN des Keystores als Character-Array
 * im Arbeitsspeicher, sie wird als String bergeben. Es wird Anwendern empfohlen,
 * eigene Implementierungen zu schreiben, die die PIN in der Methode sign(...)
 * abfragen und nach Gebrauch wieder lschen oder anderweitig fr ein sicheres
 * Pin-Cashing zu sorgen.
 *
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Organisation: </p>
 * @author R. Lindemann, N.Bngener
 * @version 1.0
 * @see de.osci.osci12.extinterfaces.crypto.Signer
 */
public class PKCS12Signer extends de.osci.osci12.extinterfaces.crypto.Signer
{
  private static Log log = LogFactory.getLog(PKCS12Signer.class);
  private X509Certificate cert;
  private String alias;
  private char[] pin = null;
  private KeyStore keyStore = null;


  public PKCS12Signer(String p12_fileName, String pin) throws Exception
  {
    this.pin = pin.toCharArray();
    keyStore = java.security.KeyStore.getInstance("PKCS12");
    keyStore.load(getClass().getResourceAsStream(p12_fileName), this.pin);
    this.alias = (String) keyStore.aliases().nextElement();
    cert = (X509Certificate) keyStore.getCertificate(alias);
  }

  /**
   * Liefert die Versionsnummer
   * @return Versionsnummer
   */
  public String getVendor(){return "BOS";}

  /**
   * Sollte den Namen des Herstellers zurckgeben.
   * @return Herstellername
   */
  public String getVersion(){return "0.9";}

  /**
   *  Liefert das Signaturzertifikat.
   *
   *@return  das Signaturzertifikat
   */
  public X509Certificate getCertificate()
  {
     return cert;
  }

  /**
   *  Signiert das bergebene Byte-Array
   *
   *@param  die zu signierenden Daten
   *@return die Signatur
   */
  public byte[] sign(byte[] data, String algorithm) throws SignatureException
  {
    if (!algorithm.equals(de.osci.osci12.common.Constants.SIGNATURE_ALGORITHM_RSA_SHA1))
      throw new UnsupportedOperationException("Signatur-Algorithmus " + algorithm +
                                              " wird von der Signer-Implementierung " + getClass() +
                                              " nicht untersttzt !");

    try
    {
      java.security.Signature sigengine = java.security.Signature.getInstance((String)Constants.JCA_JCE_MAP.get(Constants.SIGNATURE_ALGORITHM_RSA_SHA1));
      sigengine.initSign(((java.security.PrivateKey) keyStore.getKey(alias, pin)));
      sigengine.update(data);
      log.debug("Fertig mit dem Signieren");
      return sigengine.sign();
    }
    catch (Exception ex)
    {
      throw new SignatureException("Fehler beim Signieren - "+ex.getClass()+": "+ex.getLocalizedMessage());
    }
  }

}
